@@ -6,6 +6,7 @@
  */
 
 void draw_desktop(void);
+void about(void);
 void printipcerr(void);
 void printkeyhelp(char *keytext, char *desc, WINDOW * win, int highattr,
 		  int textattr);
@@ -16,6 +17,8 @@ void scrollkeyhelp(void);
 void stdexitkeyhelp(void);
 void indicate(char *message);
 void printlargenum(unsigned long long i, WINDOW * win);
+void print_packet_drops(unsigned long count, WINDOW *win, int x);
+void set_next_screen_update(struct timespec *next_screen_update, struct timespec *now);
 void infobox(char *text, char *prompt);
 void standardcolors(int color);
 void show_sort_statwin(WINDOW **, PANEL **);
