@@ -67,7 +67,7 @@ int dev_up(char *iface)
 
 	fd = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
 
-	strcpy(ifr.ifr_name, iface);
+	ifname_copy(ifr.ifr_name, iface);
 	ir = ioctl(fd, SIOCGIFFLAGS, &ifr);
 
 	close(fd);
@@ -90,7 +90,7 @@ int dev_get_ifindex(const char *iface)
 		return fd;
 
 	struct ifreq ifr;
-	strcpy(ifr.ifr_name, iface);
+	ifname_copy(ifr.ifr_name, iface);
 	int ir = ioctl(fd, SIOCGIFINDEX, &ifr);
 
 	/* need to preserve errno across call to close() */
@@ -114,7 +114,7 @@ int dev_get_mtu(const char *iface)
 		return fd;
 
 	struct ifreq ifr;
-	strcpy(ifr.ifr_name, iface);
+	ifname_copy(ifr.ifr_name, iface);
 	int ir = ioctl(fd, SIOCGIFMTU, &ifr);
 
 	/* need to preserve errno across call to close() */
@@ -138,7 +138,7 @@ int dev_get_flags(const char *iface)
 		return fd;
 
 	struct ifreq ifr;
-	strcpy(ifr.ifr_name, iface);
+	ifname_copy(ifr.ifr_name, iface);
 	int ir = ioctl(fd, SIOCGIFFLAGS, &ifr);
 
 	/* need to preserve errno across call to close() */
@@ -162,7 +162,7 @@ int dev_set_flags(const char *iface, int flags)
 		return fd;
 
 	struct ifreq ifr;
-	strcpy(ifr.ifr_name, iface);
+	ifname_copy(ifr.ifr_name, iface);
 	int ir = ioctl(fd, SIOCGIFFLAGS, &ifr);
 	if (ir == -1)
 		goto err;
@@ -190,7 +190,7 @@ int dev_clear_flags(const char *iface, int flags)
 		return fd;
 
 	struct ifreq ifr;
-	strcpy(ifr.ifr_name, iface);
+	ifname_copy(ifr.ifr_name, iface);
 	int ir = ioctl(fd, SIOCGIFFLAGS, &ifr);
 	if (ir == -1)
 		goto err;
@@ -233,11 +233,11 @@ int dev_get_ifname(int ifindex, char *ifname)
 		return ir;
 	}
 
-	strncpy(ifname, ifr.ifr_name, IFNAMSIZ);
+	ifname_copy(ifname, ifr.ifr_name);
 	return ir;
 }
 
-int dev_bind_ifindex(int fd, const int ifindex)
+static int dev_bind_ifindex(int fd, const int ifindex)
 {
 	struct sockaddr_ll fromaddr;
 	socklen_t addrlen = sizeof(fromaddr);
@@ -248,43 +248,20 @@ int dev_bind_ifindex(int fd, const int ifindex)
 	return bind(fd, (struct sockaddr *) &fromaddr, addrlen);
 }
 
-int dev_bind_ifname(int fd, const char const *ifname)
+int dev_bind_ifname(int fd, const char * const ifname)
 {
-	int ir;
-	struct ifreq ifr;
+	int ifindex = 0;
 
-	strcpy(ifr.ifr_name, ifname);
-	ir = ioctl(fd, SIOCGIFINDEX, &ifr);
-	if (ir)
-		return ir;
+	if (ifname) {
+		int ir;
+		struct ifreq ifr;
 
-	return dev_bind_ifindex(fd, ifr.ifr_ifindex);
-}
-
-char *gen_iface_msg(char *ifptr)
-{
-	static char if_msg[20];
-
-	if (ifptr == NULL)
-		strcpy(if_msg, "all interfaces");
-	else
-		strncpy(if_msg, ifptr, 20);
-
-	return if_msg;
-}
-
-
-int dev_promisc_flag(const char *dev_name)
-{
-	int flags = dev_get_flags(dev_name);
-	if (flags < 0) {
-		write_error("Unable to obtain interface parameters for %s",
-			    dev_name);
-		return -1;
+		ifname_copy(ifr.ifr_name, ifname);
+		ir = ioctl(fd, SIOCGIFINDEX, &ifr);
+		if (ir)
+			return ir;
+		ifindex = ifr.ifr_ifindex;
 	}
 
-	if (flags & IFF_PROMISC)
-		return -1;
-
-	return flags;
+	return dev_bind_ifindex(fd, ifindex);
 }
