@@ -18,23 +18,38 @@ the IP traffic monitor routine.
 #include "iptraf-ng-compat.h"
 
 #include "ipfrag.h"
+#include "list.h"
 
-static struct fragent *fraglist = NULL;
-static struct fragent *fragtail = NULL;
+struct fragdescent {
+	unsigned int min;
+	unsigned int max;
+	struct fragdescent *prev_entry;
+	struct fragdescent *next_entry;
+};
+
+struct fragent {
+	struct list_head fragent_list;
+	unsigned long s_addr;
+	in_port_t s_port;
+	unsigned long d_addr;
+	in_port_t d_port;
+	unsigned int id;
+	unsigned int protocol;
+	int firstin;
+	struct fragdescent *fragdesclist;
+	struct fragdescent *fragdesctail;
+	unsigned int bcount;
+};
+
+LIST_HEAD(frag_head);
 
 static struct fragent *addnewdgram(struct iphdr *packet)
 {
 	struct fragent *ptmp;
 
 	ptmp = xmallocz(sizeof(struct fragent));
-	if (fraglist == NULL) {
-		fraglist = ptmp;
-		ptmp->prev_entry = NULL;
-	}
-	if (fragtail != NULL) {
-		fragtail->next_entry = ptmp;
-		ptmp->prev_entry = fragtail;
-	}
+	list_add_tail(&ptmp->fragent_list, &frag_head);
+
 	ptmp->fragdesclist = xmalloc(sizeof(struct fragdescent));
 	ptmp->fragdesclist->min = 0;
 	ptmp->fragdesclist->max = 65535;
@@ -42,9 +57,6 @@ static struct fragent *addnewdgram(struct iphdr *packet)
 	ptmp->fragdesclist->prev_entry = NULL;
 	ptmp->fragdesctail = ptmp->fragdesclist;
 
-	fragtail = ptmp;
-	ptmp->next_entry = NULL;
-
 	ptmp->s_addr = packet->saddr;
 	ptmp->d_addr = packet->daddr;
 	ptmp->protocol = packet->protocol;
@@ -76,14 +88,12 @@ static struct fragdescent *addnewhole(struct fragent *frag)
 static struct fragent *searchfrags(unsigned long saddr, unsigned long daddr,
 				   unsigned int protocol, unsigned int id)
 {
-	struct fragent *ftmp = fraglist;
+	struct fragent *ftmp;
 
-	while (ftmp != NULL) {
+	list_for_each_entry(ftmp, &frag_head, fragent_list) {
 		if ((saddr == ftmp->s_addr) && (daddr == ftmp->d_addr)
 		    && (protocol == ftmp->protocol) && (id == ftmp->id))
 			return ftmp;
-
-		ftmp = ftmp->next_entry;
 	}
 
 	return NULL;
@@ -91,61 +101,31 @@ static struct fragent *searchfrags(unsigned long saddr, unsigned long daddr,
 
 static void deldgram(struct fragent *ftmp)
 {
-	if (ftmp->prev_entry != NULL)
-		ftmp->prev_entry->next_entry = ftmp->next_entry;
-	else
-		fraglist = ftmp->next_entry;
-
-	if (ftmp->next_entry != NULL)
-		ftmp->next_entry->prev_entry = ftmp->prev_entry;
-	else
-		fragtail = ftmp->prev_entry;
-
+	list_del(&ftmp->fragent_list);
 	free(ftmp);
 }
 
-
-/*
- * Destroy hole descriptor list
- */
-
+/* destroy hole descriptor list */
 static void destroyholes(struct fragent *ftmp)
 {
 	struct fragdescent *dtmp = ftmp->fragdesclist;
-	struct fragdescent *ntmp = NULL;
 
-	if (ftmp->fragdesclist != NULL) {
-		ntmp = dtmp->next_entry;
+	while (dtmp != NULL) {
+		struct fragdescent *ntmp = dtmp->next_entry;
 
-		while (dtmp != NULL) {
-			free(dtmp);
-			dtmp = ntmp;
-
-			if (ntmp != NULL)
-				ntmp = ntmp->next_entry;
-		}
+		free(dtmp);
+		dtmp = ntmp;
 	}
 }
 
 void destroyfraglist(void)
 {
-	struct fragent *ptmp = fraglist;
-	struct fragent *ctmp = NULL;
+	struct fragent *entry, *tmp;
 
-	if (fraglist != NULL) {
-		ctmp = ptmp->next_entry;
-
-		while (ptmp != NULL) {
-			destroyholes(ptmp);
-			free(ptmp);
-			ptmp = ctmp;
-
-			if (ctmp != NULL)
-				ctmp = ctmp->next_entry;
-		}
+	list_for_each_entry_safe(entry, tmp, &frag_head, fragent_list) {
+		destroyholes(entry);
+		deldgram(entry);
 	}
-	fraglist = NULL;
-	fragtail = NULL;
 }
 
 /*
@@ -183,10 +163,10 @@ unsigned int processfragment(struct iphdr *packet, in_port_t *sport,
 	 */
 
 	dtmp = ftmp->fragdesclist;	/* Point to hole descriptors */
-	offset = (ntohs(packet->frag_off) & 0x1fff) * 8;
+	offset = ipv4_frag_offset(packet);
 	lastbyte = (offset + (ntohs(packet->tot_len) - (packet->ihl) * 4)) - 1;
 
-	if ((ntohs(packet->frag_off) & 0x1fff) == 0) {	/* first fragment? */
+	if (ipv4_is_first_fragment(packet)) {	/* first fragment? */
 		ftmp->firstin = 1;
 		tpacket = ((char *) (packet)) + (packet->ihl * 4);
 		if (packet->protocol == IPPROTO_TCP) {
@@ -235,7 +215,7 @@ unsigned int processfragment(struct iphdr *packet, in_port_t *sport,
 			ntmp->max = offset - 1;
 		}
 		if ((lastbyte < dtmp->max)
-		    && (ntohs(packet->frag_off) & 0x2000)) {
+		    && ipv4_more_fragments(packet)) {
 			/*
 			 * If last byte in fragment is less than the last byte of the
 			 * hole descriptor, and more fragments, create a new hole
