@@ -67,7 +67,7 @@ static void clearfiles(char *prefix, char *directory)
 {
 	DIR *dir;
 	struct dirent *dir_entry;
-	char target_name[80];
+	char target_name[PATH_MAX];
 
 	dir = opendir(directory);
 
@@ -83,8 +83,9 @@ static void clearfiles(char *prefix, char *directory)
 		if (dir_entry != NULL) {
 			if (strncmp(dir_entry->d_name, prefix, strlen(prefix))
 			    == 0) {
-				snprintf(target_name, 80, "%s/%s", directory,
-					 dir_entry->d_name);
+				snprintf(target_name, sizeof(target_name) - 1,
+					 "%s/%s", directory, dir_entry->d_name);
+				target_name[sizeof(target_name) - 1] = '\0';
 				unlink(target_name);
 			}
 		}
@@ -284,6 +285,12 @@ static struct options iptraf_ng_options[] = {
 	OPT_STRING('L', NULL, &L_opt, "logfile",
 		   "specifies an alternate log file"),
 	//    OPT_INTEGER('I', NULL, &I_opt, "the log interval for all facilities except the IP traffic monitor. Value is in minutes"),
+// PHIL  From manual:
+// Sets the logging interval (in minutes) when the -L parameter is used. This over-
+// rides the Log interval... setting in the Configure... menu. If omitted, the configured
+// value is used. This parameter is ignored when the -L parameter is omitted and
+// logging is disabled.
+// The value specified here will affect all facilities except for the IP traffic monitor.
 	OPT_END()
 };
 
@@ -381,6 +388,15 @@ int main(int argc, char **argv)
 	if (__builtin_popcount(command) > 1)
 		die("only one of -i|-d|-s|-z|-l|-g options must be used");
 
+	/* sanity check of passed arguments */
+	if ((i_opt && strlen(i_opt) >= IFNAMSIZ) ||
+	    (d_opt && strlen(d_opt) >= IFNAMSIZ) ||
+	    (s_opt && strlen(s_opt) >= IFNAMSIZ) ||
+	    (z_opt && strlen(z_opt) >= IFNAMSIZ) ||
+	    (l_opt && strlen(l_opt) >= IFNAMSIZ)) {
+		die("interface name is too long");
+	}
+
 	strcpy(current_logfile, "");
 
 	if (f_opt) {
@@ -427,9 +443,7 @@ int main(int argc, char **argv)
 
 
 	if (create_pidfile() < 0)
-		goto cleanup;
-
-	int pidfile_created = 1;
+		goto bailout;
 
 	/*
 	 * If a facility is directly invoked from the command line, check for
@@ -458,6 +472,7 @@ int main(int argc, char **argv)
 	sanitize_dir(LOCKDIR);
 	sanitize_dir(WORKDIR);
 
+	setlocale(LC_ALL, "");	/* needed to properly init (n)curses library */
 	initscr();
 
 	if ((LINES < 24) || (COLS < 80)) {
@@ -537,8 +552,7 @@ int main(int argc, char **argv)
 	endwin();
 
 cleanup:
-	if (pidfile_created)
-		unlink(IPTRAF_PIDFILE);
-
+	unlink(IPTRAF_PIDFILE);
+bailout:
 	return 0;
 }
