@@ -19,7 +19,6 @@ options.c - implements the configuration section of the utility
 #include "deskman.h"
 #include "attrs.h"
 #include "landesc.h"
-#include "promisc.h"
 #include "dirs.h"
 
 #define ALLOW_ZERO 1
@@ -58,7 +57,7 @@ static void makeoptionmenu(struct MENU *menu)
 	tx_additem(menu, NULL, NULL);
 	tx_additem(menu, " ^E^thernet/PLIP host descriptions...",
 		   "Manages descriptions for Ethernet and PLIP addresses");
-	tx_additem(menu, " ^F^DDI/Token Ring host descriptions...",
+	tx_additem(menu, " ^F^DDI host descriptions...",
 		   "Manages descriptions for FDDI and FDDI addresses");
 	tx_additem(menu, NULL, NULL);
 	tx_additem(menu, " E^x^it configuration", "Returns to main menu");
@@ -180,10 +179,10 @@ void loadoptions(void)
 static void updatetimes(WINDOW *win)
 {
 	wattrset(win, HIGHATTR);
-	mvwprintw(win, 10, 25, "%3u mins", options.timeout);
-	mvwprintw(win, 11, 25, "%3u mins", options.logspan / 60);
-	mvwprintw(win, 12, 25, "%3u secs", options.updrate);
-	mvwprintw(win, 13, 25, "%3u mins", options.closedint);
+	mvwprintw(win, 10, 25, "%3ld mins", options.timeout);
+	mvwprintw(win, 11, 25, "%3ld mins", options.logspan / 60);
+	mvwprintw(win, 12, 25, "%3ld secs", options.updrate);
+	mvwprintw(win, 13, 25, "%3ld mins", options.closedint);
 }
 
 static void showoptions(WINDOW *win)
@@ -212,8 +211,7 @@ static void settimeout(time_t *value, const char *units, int allow_zero,
 	tx_box(dlgwin, ACS_VLINE, ACS_HLINE);
 
 	wattrset(dlgwin, DLGTEXTATTR);
-	wmove(dlgwin, 2, 2);
-	wprintw(dlgwin, "Enter value in %s", units);
+	mvwprintw(dlgwin, 2, 2, "Enter value in %s", units);
 	wmove(dlgwin, 5, 2);
 	stdkeyhelp(dlgwin);
 
@@ -380,7 +378,7 @@ void setoptions(void)
 		indicatesetting(row, statwin);
 	} while (row != 18);
 
-	destroyporttab(ports);
+	destroyporttab(&ports);
 	tx_destroymenu(&menu);
 	del_panel(statpanel);
 	delwin(statwin);
