@@ -10,12 +10,10 @@ struct OPTIONS {
 	time_t closedint;
 };
 
-#define DEFAULT_UPDATE_DELAY 50000	/* usec screen delay if update rate 0 */
+extern struct OPTIONS options;
 
-#include "serv.h"
-
-void setoptions(struct OPTIONS *options, struct porttab **ports);
-void loadoptions(struct OPTIONS *options);
-void saveoptions(struct OPTIONS *options);
+void setoptions(void);
+void loadoptions(void);
+void saveoptions(void);
 
 #endif	/* IPTRAF_NG_OPTIONS_H */
