@@ -12,11 +12,11 @@ tcptable.c - table manipulation routines for the IP monitor
 #include "tui/winops.h"
 
 #include "options.h"
+#include "revname.h"
 #include "tcptable.h"
 #include "deskman.h"
 #include "attrs.h"
 #include "log.h"
-#include "revname.h"
 #include "rvnamed.h"
 #include "servname.h"
 #include "hostmon.h"
@@ -33,17 +33,13 @@ static void setlabels(WINDOW *win, int mode)
 	whline(win, ACS_HLINE, 23 * COLS / 80);
 
 	if (mode == 0) {
-		wmove(win, 0, 47 * COLS / 80);
-		wprintw(win, " Packets ");
-		wmove(win, 0, 59 * COLS / 80);
-		wprintw(win, " Bytes ");
+		mvwprintw(win, 0, 47 * COLS / 80, " Packets ");
+		mvwprintw(win, 0, 59 * COLS / 80, " Bytes ");
 	} else if (mode == 1) {
 		mvwprintw(win, 0, 47 * COLS / 80, " Source MAC Addr ");
 	} else if (mode == 2) {
-		wmove(win, 0, 45 * COLS / 80);
-		wprintw(win, " Pkt Size ");
-		wmove(win, 0, 56 * COLS / 80);
-		wprintw(win, " Win Size ");
+		mvwprintw(win, 0, 45 * COLS / 80, " Pkt Size ");
+		mvwprintw(win, 0, 56 * COLS / 80, " Win Size ");
 	}
 }
 
@@ -58,7 +54,7 @@ static unsigned int tcp_hash(struct sockaddr_storage *saddr,
 	size_t i;
 	unsigned int ifsum = 0;
 
-	for (i = 0; i <= strlen(ifname) - 1; i++)
+	for (i = 0; i < strlen(ifname); i++)
 		ifsum += ifname[i];
 
 	switch (saddr->ss_family) {
@@ -100,6 +96,20 @@ static void print_tcp_num_entries(struct tcptable *table)
 		  table->count);
 }
 
+static void prepare_statwin(WINDOW *win)
+{
+	wattrset(win, IPSTATLABELATTR);
+	mvwprintw(win, 0, 1, "Packets captured:");
+	mvwaddch(win, 0, 45 * COLS / 80, ACS_VLINE);
+}
+
+void show_stats(WINDOW *win, unsigned long long total)
+{
+	wattrset(win, IPSTATATTR);
+	wmove(win, 0, 35 * COLS / 80);
+	printlargenum(total, win);
+}
+
 void init_tcp_table(struct tcptable *table)
 {
 	int i;
@@ -112,15 +122,13 @@ void init_tcp_table(struct tcptable *table)
 
 	wattrset(table->borderwin, BOXATTR);
 	tx_box(table->borderwin, ACS_VLINE, ACS_HLINE);
-	wmove(table->borderwin, 0, 1);
-	wprintw(table->borderwin, " TCP Connections (Source Host:Port) ");
+	mvwprintw(table->borderwin, 0, 1, " TCP Connections (Source Host:Port) ");
 
 	setlabels(table->borderwin, 0);	/* initially use mode 0 */
+	table->mode = 0;
 
-	wmove(table->borderwin, 0, 65 * COLS / 80);
-	wprintw(table->borderwin, " Flag ");
-	wmove(table->borderwin, 0, 70 * COLS / 80);
-	wprintw(table->borderwin, " Iface ");
+	mvwprintw(table->borderwin, 0, 65 * COLS / 80, " Flag ");
+	mvwprintw(table->borderwin, 0, 70 * COLS / 80, " Iface ");
 	update_panels();
 	doupdate();
 	table->ifnamew = COLS - (70 * COLS / 80) - 3;
@@ -143,6 +151,13 @@ void init_tcp_table(struct tcptable *table)
 	tx_stdwinset(table->tcpscreen);
 	print_tcp_num_entries(table);
 
+	table->statwin = newwin(1, COLS, LINES - 2, 0);
+	table->statpanel = new_panel(table->statwin);
+	wattrset(table->statwin, IPSTATLABELATTR);
+	mvwprintw(table->statwin, 0, 0, "%*c", COLS, ' ');
+	prepare_statwin(table->statwin);
+	show_stats(table->statwin, 0);
+
 	/* 
 	 * Initialize hash table to nulls
 	 */
@@ -152,7 +167,6 @@ void init_tcp_table(struct tcptable *table)
 		table->hash_tails[i] = NULL;
 	}
 	table->barptr = NULL;
-	table->baridx = 0;
 }
 
 /*
@@ -224,15 +238,30 @@ static void del_tcp_hash_node(struct tcptable *table, struct tcptableent *entry)
 	free(ptmp);
 }
 
+static void resolve_entry(struct tcptableent *entry, struct resolver *res)
+{
+	if (entry->s_fstat != RESOLVED) {
+		entry->s_fstat = revname(res, &entry->saddr,
+					 entry->s_fqdn, sizeof(entry->s_fqdn));
+		strcpy(entry->oth_connection->d_fqdn, entry->s_fqdn);
+		entry->oth_connection->d_fstat = entry->s_fstat;
+	}
+	if (entry->d_fstat != RESOLVED) {
+		entry->d_fstat = revname(res, &entry->daddr,
+					 entry->d_fqdn, sizeof(entry->d_fqdn));
+		strcpy(entry->oth_connection->s_fqdn, entry->d_fqdn);
+		entry->oth_connection->s_fstat = entry->d_fstat;
+	}
+}
+
 /*
  * Add a new entry to the TCP screen table
  */
-
 struct tcptableent *addentry(struct tcptable *table,
 			     struct sockaddr_storage *saddr,
 			     struct sockaddr_storage *daddr,
 			     int protocol, char *ifname,
-			     int *rev_lookup, int rvnfd)
+			     struct resolver *res)
 {
 	struct tcptableent *new_entry;
 	struct closedlist *ctemp;
@@ -252,6 +281,7 @@ struct tcptableent *addentry(struct tcptable *table,
 			table->head = new_entry;
 
 			table->firstvisible = new_entry;
+			table->barptr = new_entry;
 		}
 		if (table->tail != NULL) {
 			table->tail->next_entry = new_entry;
@@ -275,13 +305,10 @@ struct tcptableent *addentry(struct tcptable *table,
 			 table->firstvisible->index + (table->imaxy - 1))
 			table->lastvisible = new_entry;
 
-		new_entry->reused = new_entry->oth_connection->reused = 0;
 		table->count++;
 
 		rate_alloc(&new_entry->rate, 5);
 		rate_alloc(&new_entry->oth_connection->rate, 5);
-
-		print_tcp_num_entries(table);
 	} else {
 		/*
 		 * If we reach this point, we're allocating off the list of closed
@@ -305,8 +332,6 @@ struct tcptableent *addentry(struct tcptable *table,
 		if (table->closedentries == NULL)
 			table->closedtail = NULL;
 
-		new_entry->reused = new_entry->oth_connection->reused = 1;
-
 		/*
 		 * Delete the old hash entries for this reallocated node;
 		 */
@@ -340,8 +365,8 @@ struct tcptableent *addentry(struct tcptable *table,
 	 * Store interface name
 	 */
 
-	strcpy(new_entry->ifname, ifname);
-	strcpy(new_entry->oth_connection->ifname, ifname);
+	ifname_copy(new_entry->ifname, ifname);
+	ifname_copy(new_entry->oth_connection->ifname, ifname);
 
 	/*
 	 * Zero out MAC address fields
@@ -352,13 +377,9 @@ struct tcptableent *addentry(struct tcptable *table,
 
 	new_entry->stat = new_entry->oth_connection->stat = 0;
 
-	new_entry->s_fstat =
-	    revname(rev_lookup, &new_entry->saddr,
-		    new_entry->s_fqdn, sizeof(new_entry->s_fqdn), rvnfd);
-
-	new_entry->d_fstat =
-	    revname(rev_lookup, &new_entry->daddr,
-		    new_entry->d_fqdn, sizeof(new_entry->d_fqdn), rvnfd);
+	new_entry->s_fstat = NOTRESOLVED;
+	new_entry->d_fstat = NOTRESOLVED;
+	resolve_entry(new_entry, res);
 
 	/* set port service names (where applicable) */
 	servlook(sockaddr_get_port(saddr), IPPROTO_TCP, new_entry->s_sname, 10);
@@ -395,13 +416,10 @@ struct tcptableent *addentry(struct tcptable *table,
 	 * Mark flow rate start time and byte counter for flow computation
 	 * if the highlight bar is on either flow of the new connection.
 	 */
-	if (table->barptr == new_entry) {
-		new_entry->starttime = time(NULL);
+	if (table->barptr == new_entry)
 		new_entry->spanbr = 0;
-	} else if (table->barptr == new_entry->oth_connection) {
-		new_entry->oth_connection->starttime = time(NULL);
+	else if (table->barptr == new_entry->oth_connection)
 		new_entry->oth_connection->spanbr = 0;
-	}
 
 	/*
 	 * Add entries to hash table
@@ -444,8 +462,10 @@ static char *tcplog_flowrate_msg(struct tcptableent *entry, char *buf,
 				 size_t bufsize)
 {
 	time_t interval = time(NULL) - entry->conn_starttime;
+	if (interval < 1)
+		interval = 1;
 
-	char rbuf[64];
+	char rbuf[32];
 	rate_print(entry->bcount / interval, rbuf, sizeof(rbuf));
 
 	snprintf(buf, bufsize - 1, "avg flow rate %s", rbuf);
@@ -472,19 +492,38 @@ void write_timeout_log(int logging, FILE *logfile, struct tcptableent *tcpnode)
 	}
 }
 
+void mark_timeouted_entries(struct tcptable *table, int logging, FILE *logfile)
+{
+	if (options.timeout == 0)
+		return;
+
+	time_t now = time(NULL);
+	struct tcptableent *ptmp = table->head;
+
+	while (ptmp != NULL) {
+		if (((now - ptmp->lastupdate) / 60 > options.timeout)
+		    && !ptmp->inclosed) {
+			ptmp->timedout = 1;
+			ptmp->oth_connection->timedout = 1;
+			addtoclosedlist(table, ptmp);
+
+			if (logging)
+				write_timeout_log(logging, logfile, ptmp);
+		}
+		ptmp = ptmp->next_entry;
+	}
+}
+
 struct tcptableent *in_table(struct tcptable *table,
 			     struct sockaddr_storage *saddr,
 			     struct sockaddr_storage *daddr,
-			     char *ifname, int logging,
-			     FILE *logfile, time_t timeout)
+			     char *ifname)
 {
 	struct tcp_hashentry *hashptr;
 	unsigned int hp;
 
-	time_t now;
-
 	if (table->head == NULL) {
-		return 0;
+		return NULL;
 	}
 	/*
 	 * Determine hash table index for this set of addresses and ports
@@ -499,24 +538,6 @@ struct tcptableent *in_table(struct tcptable *table,
 		    && (strcmp(hashptr->tcpnode->ifname, ifname) == 0))
 			break;
 
-		now = time(NULL);
-
-		/*
-		 * Add the timed out entries to the closed list in case we didn't
-		 * find any closed ones.
-		 */
-
-		if ((timeout > 0)
-		    && ((now - hashptr->tcpnode->lastupdate) / 60 > timeout)
-		    && (!(hashptr->tcpnode->inclosed))) {
-			hashptr->tcpnode->timedout = 1;
-			hashptr->tcpnode->oth_connection->timedout = 1;
-			addtoclosedlist(table, hashptr->tcpnode);
-
-			if (logging)
-				write_timeout_log(logging, logfile,
-						  hashptr->tcpnode);
-		}
 		hashptr = hashptr->next_entry;
 	}
 
@@ -541,45 +562,28 @@ struct tcptableent *in_table(struct tcptable *table,
  * Update the TCP status record should an applicable packet arrive.
  */
 
-void updateentry(struct tcptable *table, struct tcptableent *tableentry,
-		 struct tcphdr *transpacket, char *packet, int linkproto,
-		 unsigned long packetlength, unsigned int bcount,
-		 unsigned int fragofs, int logging, int *revlook, int rvnfd,
+void updateentry(struct tcptable *table, struct pkt_hdr *pkt,
+		 struct tcptableent *tableentry, struct tcphdr *tcp,
+		 unsigned int bcount, struct resolver *res, int logging,
 		 FILE *logfile)
 {
 	char msgstring[MSGSTRING_MAX];
 	char newmacaddr[18];
 
-	if (tableentry->s_fstat != RESOLVED) {
-		tableentry->s_fstat =
-		    revname(revlook, &tableentry->saddr, tableentry->s_fqdn,
-			    sizeof(tableentry->s_fqdn), rvnfd);
-		strcpy(tableentry->oth_connection->d_fqdn, tableentry->s_fqdn);
-		tableentry->oth_connection->d_fstat = tableentry->s_fstat;
-	}
-	if (tableentry->d_fstat != RESOLVED) {
-		tableentry->d_fstat =
-		    revname(revlook, &tableentry->daddr, tableentry->d_fqdn,
-			    sizeof(tableentry->d_fqdn), rvnfd);
-		strcpy(tableentry->oth_connection->s_fqdn, tableentry->d_fqdn);
-		tableentry->oth_connection->s_fstat = tableentry->d_fstat;
-	}
+	resolve_entry(tableentry, res);
+
 	tableentry->pcount++;
 	tableentry->bcount += bcount;
-	tableentry->psize = packetlength;
+	tableentry->psize = pkt->pkt_len;
 	tableentry->spanbr += bcount;
 
 	if (options.mac) {
 		memset(newmacaddr, 0, sizeof(newmacaddr));
 
-
-		/* change updateentry to take struct pkt to remove this */
-		if (linkproto == ARPHRD_ETHER) {
-			convmacaddr((char *) (((struct ethhdr *) packet)->
-					      h_source), newmacaddr);
-		} else if (linkproto == ARPHRD_FDDI) {
-			convmacaddr((char *) (((struct fddihdr *) packet)->
-					      saddr), newmacaddr);
+		if (pkt->from->sll_hatype == ARPHRD_ETHER) {
+			convmacaddr((char *) pkt->ethhdr->h_source, newmacaddr);
+		} else if (pkt->from->sll_hatype == ARPHRD_FDDI) {
+			convmacaddr((char *) pkt->fddihdr->saddr, newmacaddr);
 		}
 
 		if (tableentry->smacaddr[0] != '\0') {
@@ -603,7 +607,7 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 	 * TCP header.
 	 */
 
-	if ((ntohs(fragofs) & 0x1fff) != 0) {
+	if (!packet_is_first_fragment(pkt)) {
 		tableentry->lastupdate =
 		    tableentry->oth_connection->lastupdate = time(NULL);
 		return;
@@ -613,19 +617,19 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 	 */
 
 	if (tableentry->pcount == 1) {
-		if ((transpacket->syn) || (transpacket->rst))
+		if (tcp->syn || tcp->rst)
 			tableentry->partial = 0;
 		else
 			tableentry->partial = 1;
 	}
-	tableentry->win = ntohs(transpacket->window);
+	tableentry->win = ntohs(tcp->window);
 
 	tableentry->stat = 0;
 
-	if (transpacket->syn)
+	if (tcp->syn)
 		tableentry->stat |= FLAG_SYN;
 
-	if (transpacket->ack) {
+	if (tcp->ack) {
 		tableentry->stat |= FLAG_ACK;
 
 		/*
@@ -639,7 +643,7 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 		 */
 
 		if ((tableentry->oth_connection->finsent == 1)
-		    && (ntohl(transpacket->seq) ==
+		    && (ntohl(tcp->seq) ==
 			tableentry->oth_connection->finack)) {
 			tableentry->oth_connection->finsent = 2;
 
@@ -660,7 +664,7 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 	 * in ACK above.
 	 */
 
-	if (transpacket->fin) {
+	if (tcp->fin) {
 
 		/*
 		 * First, we check if the opposite direction has no counts, in which
@@ -688,7 +692,7 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 			 */
 
 			tableentry->finsent = 1;
-			tableentry->finack = ntohl(transpacket->ack_seq);
+			tableentry->finack = ntohl(tcp->ack_seq);
 		}
 		if (logging) {
 			char flowrate[64];
@@ -701,7 +705,7 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 				    tableentry->psize, msgstring);
 		}
 	}
-	if (transpacket->rst) {
+	if (tcp->rst) {
 		tableentry->stat |= FLAG_RST;
 		if (!(tableentry->inclosed))
 			addtoclosedlist(table, tableentry);
@@ -720,10 +724,10 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 				    tableentry->psize, msgstring);
 		}
 	}
-	if (transpacket->psh)
+	if (tcp->psh)
 		tableentry->stat |= FLAG_PSH;
 
-	if (transpacket->urg)
+	if (tcp->urg)
 		tableentry->stat |= FLAG_URG;
 
 	tableentry->lastupdate = tableentry->oth_connection->lastupdate =
@@ -746,45 +750,22 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 
 }
 
-/*
- * Clears out the resolved IP addresses from the window.  This prevents
- * overlapping port numbers (in cases where the resolved DNS name is shorter
- * than its IP address), that may cause the illusion of large ports.  Plus,
- * such output, while may be interpreted by people with a little know-how,
- * is just plain wrong.
- *
- * Returns immediately if the entry is not visible in the window.
- */
-
-void clearaddr(struct tcptable *table, struct tcptableent *tableentry,
-	       unsigned int screen_idx)
-{
-	unsigned int target_row;
-
-	if ((tableentry->index < screen_idx)
-	    || (tableentry->index > screen_idx + (table->imaxy - 1)))
-		return;
-
-	target_row = (tableentry->index) - screen_idx;
-
-	wmove(table->tcpscreen, target_row, 1);
-	wprintw(table->tcpscreen, "%44c", ' ');
-}
-
 /*
  * Display a TCP connection line.  Returns immediately if the entry is
  * not visible in the window.
  */
 
-void printentry(struct tcptable *table, struct tcptableent *tableentry,
-		unsigned int screen_idx, int mode)
+void printentry(struct tcptable *table, struct tcptableent *tableentry)
 {
 	char stat[7] = "";
 	unsigned int target_row;
-	char sp_buf[MSGSTRING_MAX];
 	int normalattr;
 	int highattr;
 
+	if ((tableentry->index < table->firstvisible->index)
+	    || (tableentry->index > table->lastvisible->index))
+		return;
+
 	/*
 	 * Set appropriate attributes for this entry
 	 */
@@ -797,40 +778,25 @@ void printentry(struct tcptable *table, struct tcptableent *tableentry,
 		highattr = HIGHATTR;
 	}
 
-	if ((tableentry->index < screen_idx)
-	    || (tableentry->index > screen_idx + (table->imaxy - 1)))
-		return;
+	target_row = tableentry->index - table->firstvisible->index;
 
-	target_row = (tableentry->index) - screen_idx;
+	/* clear the target line */
+	wattrset(table->tcpscreen, normalattr);
+	scrollok(table->tcpscreen, 0);
+	mvwprintw(table->tcpscreen, target_row, 0, "%*c", COLS - 2, ' ');
+	scrollok(table->tcpscreen, 1);
 
-	/* clear the data if it's a reused entry */
-
-	wattrset(table->tcpscreen, PTRATTR);
-	wmove(table->tcpscreen, target_row, 2);
-	if (tableentry->reused) {
-		scrollok(table->tcpscreen, 0);
-		sprintf(sp_buf, "%%%dc", COLS - 4);
-		wprintw(table->tcpscreen, sp_buf, ' ');
-		scrollok(table->tcpscreen, 1);
-		tableentry->reused = 0;
-		wmove(table->tcpscreen, target_row, 1);
-	}
 	/* print half of connection indicator bracket */
-
-	wmove(table->tcpscreen, target_row, 0);
-	waddch(table->tcpscreen, tableentry->half_bracket);
+	wattrset(table->tcpscreen, PTRATTR);
+	mvwaddch(table->tcpscreen, target_row, 0, tableentry->half_bracket);
 
 	/* proceed with the actual entry */
-
 	wattrset(table->tcpscreen, normalattr);
-	sprintf(sp_buf, "%%%dc", COLS - 5);
-	mvwprintw(table->tcpscreen, target_row, 2, sp_buf, ' ');
-
-	sprintf(sp_buf, "%%.%ds:%%.%ds", 32 * COLS / 80, 10);
-
-	wmove(table->tcpscreen, target_row, 1);
-	wprintw(table->tcpscreen, sp_buf, tableentry->s_fqdn,
-		tableentry->s_sname);
+	mvwprintw(table->tcpscreen,
+		  target_row, 1,
+		  "%.*s:%.*s",
+		  32 * COLS / 80, tableentry->s_fqdn,
+		  10, tableentry->s_sname);
 
 	wattrset(table->tcpscreen, highattr);
 
@@ -839,16 +805,16 @@ void printentry(struct tcptable *table, struct tcptableent *tableentry,
 	 * on the value of mode.
 	 */
 
-	switch (mode) {
+	switch (table->mode) {
 	case 0:
 		wmove(table->tcpscreen, target_row, 47 * COLS / 80 - 2);
 		if (tableentry->partial)
 			wprintw(table->tcpscreen, ">");
 		else
 			wprintw(table->tcpscreen, "=");
-		wprintw(table->tcpscreen, "%8u  ", tableentry->pcount);
+		printlargenum(tableentry->pcount, table->tcpscreen);
 		wmove(table->tcpscreen, target_row, 59 * COLS / 80 - 4);
-		wprintw(table->tcpscreen, "%9u  ", tableentry->bcount);
+		printlargenum(tableentry->bcount, table->tcpscreen);
 		break;
 	case 1:
 		wmove(table->tcpscreen, target_row, 50 * COLS / 80);
@@ -866,7 +832,9 @@ void printentry(struct tcptable *table, struct tcptableent *tableentry,
 
 	wattrset(table->tcpscreen, normalattr);
 
-	if (tableentry->finsent == 1)
+	if (tableentry->timedout)
+		strcpy(stat, "TMOU");
+	else if (tableentry->finsent == 1)
 		strcpy(stat, "DONE");
 	else if (tableentry->finsent == 2)
 		strcpy(stat, "CLOS");
@@ -886,25 +854,38 @@ void printentry(struct tcptable *table, struct tcptableent *tableentry,
 		tableentry->ifname);
 }
 
+void resolve_visible_entries(struct tcptable *table, struct resolver *res)
+{
+
+	if(res->lookup == false)
+		return;
+
+	struct tcptableent *entry = table->firstvisible;
+	while ((entry != NULL) && (entry->prev_entry != table->lastvisible)) {
+		resolve_entry(entry, res);
+
+		entry = entry->next_entry;
+	}
+}
+
 /*
  * Redraw the TCP window
  */
-
-void refreshtcpwin(struct tcptable *table, unsigned int idx, int mode)
+void refreshtcpwin(struct tcptable *table, bool clear)
 {
 	struct tcptableent *ptmp;
 
-	setlabels(table->borderwin, mode);
-	wattrset(table->tcpscreen, STDATTR);
-	tx_colorwin(table->tcpscreen);
-	ptmp = table->firstvisible;
-
-	while ((ptmp != NULL) && (ptmp->prev_entry != table->lastvisible)) {
-		printentry(table, ptmp, idx, mode);
-		ptmp = ptmp->next_entry;
+	if (clear) {
+		setlabels(table->borderwin, table->mode);
+		wattrset(table->tcpscreen, STDATTR);
+		tx_colorwin(table->tcpscreen);
 	}
 
-	wmove(table->borderwin, table->bmaxy - 1, 1);
+	ptmp = table->firstvisible;
+	while ((ptmp != NULL) && (ptmp->prev_entry != table->lastvisible)) {
+		printentry(table, ptmp);
+		ptmp = ptmp->next_entry;
+	}
 
 	print_tcp_num_entries(table);
 
@@ -914,79 +895,65 @@ void refreshtcpwin(struct tcptable *table, unsigned int idx, int mode)
 
 static void destroy_closed_entries(struct tcptable *table)
 {
-	struct closedlist *closedtemp;
-	struct closedlist *closedtemp_next;
+	struct closedlist *ptmp = table->closedentries;
 
-	if (table->closedentries != NULL) {
-		closedtemp = table->closedentries;
-		closedtemp_next = table->closedentries->next_entry;
+	while (ptmp != NULL) {
+		struct closedlist *ctmp = ptmp->next_entry;
 
-		while (closedtemp != NULL) {
-			free(closedtemp);
-			closedtemp = closedtemp_next;
-			if (closedtemp_next != NULL)
-				closedtemp_next = closedtemp_next->next_entry;
+		free(ptmp);
+		ptmp = ctmp;
+	}
+	table->closedentries = NULL;
+	table->closedtail = NULL;
+}
+
+static void destroy_tcp_entries(struct tcptable *table)
+{
+	struct tcptableent *ptmp = table->head;
+
+	while (ptmp != NULL) {
+		struct tcptableent *ctmp = ptmp->next_entry;
+
+		rate_destroy(&ptmp->rate);
+		free(ptmp);
+		ptmp = ctmp;
+	}
+	table->head = NULL;
+	table->tail = NULL;
+}
+
+static void destroy_hash_entries(struct tcptable *table)
+{
+	for (unsigned int i = 0; i < ENTRIES_IN_HASH_TABLE; i++) {
+		struct tcp_hashentry *ptmp = table->hash_table[i];
+		while (ptmp != NULL) {
+			struct tcp_hashentry *ctmp = ptmp->next_entry;
+
+			free(ptmp);
+			ptmp = ctmp;
 		}
-
-		table->closedentries = NULL;
-		table->closedtail = NULL;
+		table->hash_table[i] = NULL;
 	}
 }
 
-/*
- * Kill the entire TCP table
- */
+/* kill the entire TCP table */
 void destroytcptable(struct tcptable *table)
 {
-	struct tcptableent *ctemp;
-	struct tcptableent *c_next_entry;
-	struct tcp_hashentry *hashtemp;
-	struct tcp_hashentry *hashtemp_next;
-
-	unsigned int i;
-
-	/*
-	 * Destroy main TCP table
-	 */
-
-	if (table->head != NULL) {
-		ctemp = table->head;
-		c_next_entry = table->head->next_entry;
-
-		while (ctemp != NULL) {
-			rate_destroy(&ctemp->rate);
-			free(ctemp);
-			ctemp = c_next_entry;
-
-			if (c_next_entry != NULL)
-				c_next_entry = c_next_entry->next_entry;
-		}
-	}
-	/*
-	 * Destroy list of closed entries
-	 */
+	/* destroy main TCP table */
+	destroy_tcp_entries(table);
 
+	/* destroy list of closed entries */
 	destroy_closed_entries(table);
 
-	/*
-	 * Destroy hash table
-	 */
+	/* destroy hash table */
+	destroy_hash_entries(table);
 
-	for (i = 0; i <= ENTRIES_IN_HASH_TABLE - 1; i++) {
-		if (table->hash_table[i] != NULL) {
-			hashtemp = table->hash_table[i];
-			hashtemp_next = table->hash_table[i]->next_entry;
-
-			while (hashtemp != NULL) {
-				free(hashtemp);
-				hashtemp = hashtemp_next;
-
-				if (hashtemp_next != NULL)
-					hashtemp_next =
-					    hashtemp_next->next_entry;
-			}
-		}
-	}
+	del_panel(table->statpanel);
+	delwin(table->statwin);
+	del_panel(table->tcppanel);
+	delwin(table->tcpscreen);
+	del_panel(table->borderpanel);
+	delwin(table->borderwin);
 }
 
 /*
@@ -1019,33 +986,16 @@ static void destroy_tcp_entry(struct tcptable *table, struct tcptableent *ptmp)
  * entries.
  */
 
-void flushclosedentries(struct tcptable *table, unsigned long *screen_idx,
-			int logging, FILE *logfile)
+void flushclosedentries(struct tcptable *table)
 {
 	struct tcptableent *ptmp = table->head;
 	struct tcptableent *ctmp = NULL;
 	unsigned long idx = 1;
-	time_t now;
-	time_t lastupdated = 0;
+	unsigned long screen_idx = table->firstvisible->index;
 
 	while (ptmp != NULL) {
-		now = time(NULL);
-		lastupdated = (now - ptmp->lastupdate) / 60;
-
-		if ((ptmp->inclosed) || (lastupdated > options.timeout)) {
+		if (ptmp->inclosed) {
 			ctmp = ptmp;
-			/*
-			 * Mark and flush timed out TCP entries.
-			 */
-			if (lastupdated > options.timeout) {
-				if ((!(ptmp->timedout)) && (!(ptmp->inclosed))) {
-					write_timeout_log(logging, logfile,
-							  ptmp);
-					ptmp->timedout =
-					    ptmp->oth_connection->timedout = 1;
-				}
-			}
-
 			/*
 			 * Advance to next entry and destroy target entry.
 			 */
@@ -1077,21 +1027,21 @@ void flushclosedentries(struct tcptable *table, unsigned long *screen_idx,
 			 * Adjust screen index if the deleted entry was "above"
 			 * the screen.
 			 */
-			if (idx < *screen_idx)
-				(*screen_idx)--;
+			if (idx < screen_idx)
+				screen_idx--;
 		} else {
 			/*
 			 * Set the first visible pointer once the index matches
 			 * the screen index.
 			 */
-			if (idx == *screen_idx)
+			if (idx == screen_idx)
 				table->firstvisible = ptmp;
 
 			/*
 			 * Keep setting the last visible pointer until the scan
 			 * index "leaves" the screen
 			 */
-			if (idx <= (*screen_idx) + (table->imaxy - 1))
+			if (idx <= screen_idx + (table->imaxy - 1))
 				table->lastvisible = ptmp;
 
 			ptmp->index = idx;
@@ -1116,8 +1066,8 @@ void flushclosedentries(struct tcptable *table, unsigned long *screen_idx,
 		 * "above" the screen index.  Set the firstvisible pointer to that
 		 * as well.
 		 */
-		if (table->tail->index < *screen_idx) {
-			*screen_idx = table->tail->index;
+		if (table->tail->index < screen_idx) {
+			screen_idx = table->tail->index;
 			table->firstvisible = table->tail;
 		}
 
@@ -1128,16 +1078,11 @@ void flushclosedentries(struct tcptable *table, unsigned long *screen_idx,
 		 * head of the table.  The highlight bar should "go along" with
 		 * the shifting.
 		 */
-		while ((table->tail->index < *screen_idx + table->imaxy - 1)
+		while ((table->tail->index < screen_idx + table->imaxy - 1)
 		       && (table->firstvisible->prev_entry != NULL)) {
 			table->firstvisible = table->firstvisible->prev_entry;
-			(*screen_idx)--;
+			screen_idx--;
 		}
-
-		/*
-		 * Set the bar position index once everything's done.
-		 */
-		table->baridx = table->barptr->index - *screen_idx + 1;
 	}
 }
 
