@@ -45,13 +45,13 @@ struct MENU {
 	char shortcuts[SHORTCUTSTRLEN_MAX];
 };
 
-extern void tx_initmenu(struct MENU *menu, int y1, int x1, int y2, int x2,
-			int borderattr, int normalattr, int highattr,
-			int barnormalattr, int barhighattr, int descattr);
-extern void tx_additem(struct MENU *menu, char *item, char *desc);
-extern void tx_showitem(struct MENU *menu, struct ITEM *itemptr, int selected);
-extern void tx_showmenu(struct MENU *menu);
-extern void tx_operatemenu(struct MENU *menu, int *row, int *aborted);
-extern void tx_destroymenu(struct MENU *menu);
+void tx_initmenu(struct MENU *menu, int y1, int x1, int y2, int x2,
+		 int borderattr, int normalattr, int highattr,
+		 int barnormalattr, int barhighattr, int descattr);
+void tx_additem(struct MENU *menu, char *item, char *desc);
+void tx_showitem(struct MENU *menu, struct ITEM *itemptr, int selected);
+void tx_showmenu(struct MENU *menu);
+void tx_operatemenu(struct MENU *menu, int *row, int *aborted);
+void tx_destroymenu(struct MENU *menu);
 
 #endif	/* IPTRAF_NG_TUI_MENURT_H */
